%define grob_client_user grobclient

Summary: grob is a simple wrapper for borg-backup
Name: grob
Version: 0.1.8
Release: 1
URL: https://git.tilab.tuwien.ac.at/sw/%{name}
Source0: %{name}-%{version}.tar.xz
License: GPLv3

Requires: bash-base
Requires: bash-yaml
Requires: borgbackup
Requires: socat
Requires: expect

%description
grob is a frontend for borg

ATTENTION: This is currently in early alpha stage, do not use in
production unless you know what you do.

%package client
Summary: grob client part
Requires: openssh-server
Requires: borgbackup
Requires: socat
Requires: libcap
Requires: libcap-ng
Requires: yamllint
BuildRequires: gcc
BuildRequires: make
BuildRequires: libcap-ng-devel


%description client
Environment and helper scripts to backup a client host from a grob
backup server.

Note: It will modify /etc/pam.d/ssh to enable capabilities, so
everything can run as user %{grob_client_user} instead of root.

Make sure that your backupserver is able to connect via ssh.

%prep
%setup -q

%build
make -C client-cap-wrapper

%install
rm -rf %{buildroot}
mkdir -p %{buildroot}/{%{_bindir},%{_libexecdir}/%{name}}
cp bin/* %{buildroot}/%{_bindir}/.
cp libexec/grob-callback-handler %{buildroot}/%{_libexecdir}/.
cp libexec/grob-client* %{buildroot}/%{_libexecdir}/%{name}/.
cp client-cap-wrapper/grob-client-borg-cap-wrapper \
   %{buildroot}/%{_libexecdir}/%{name}/.

%clean
rm -rf %{buildroot}

%pre client
getent passwd %{grob_client_user} >/dev/null || {
    useradd --system --create-home \
        --home-dir /home/%{grob_client_user} \
        --comment "grob backup client user" \
        %{grob_client_user}
}

%post client
# enable pam_cap for sshd
if ! grep -qE "^[[:blank:]]*auth[[:blank:]]+required[[:blank:]]+pam_cap.so" \
    /etc/pam.d/sshd
then
    echo "auth required pam_cap.so # controlled_by_grob-client (remove comment to take over manually)" >> /etc/pam.d/sshd
fi

# apply capabilities to borg
setcap cap_dac_read_search+ie \
       /%{_libexecdir}/%{name}/grob-client-borg-cap-wrapper

# create/modify capability.conf
if [ -f /etc/security/capability.conf ]
then
    if ! grep -qE '^[[:blank:]]*cap_dac_read_search[[:blank:]]+%{grob_client_user}[[:blank:]]*$' /etc/security/capability.conf
    then
      sed -i '1i cap_dac_read_search %{grob_client_user}' \\
          /etc/security/capability.conf
    fi
else
    cat > /etc/security/capability.conf << EOF
# this file is controlled_by_grob-client (remove comment to take over manually)
# allow backup user to read and search all files with applications that have that cap
cap_dac_read_search %{grob_client_user}

# no rights by default
none *
EOF
fi

%postun client
if [[ "$1" == 0 ]]
then # uninstall

    # remove us from pam.d/sshd if we control entry
    if grep -qE "^[[:blank:]]*auth[[:blank:]]+required[[:blank:]]+pam_cap.so.*controlled_by_grob-client" \
        /etc/pam.d/sshd
    then
        sed -ri '/^[[:blank:]]*auth[[:blank:]]+required[[:blank:]]+pam_cap.so[[:blank:]]*#[[:blank:]]*controlled_by_grob-client/ d' /etc/pam.d/sshd
    fi

    # remove/modify capability.conf
    if [ -f /etc/security/capability.conf ]
    then
        if grep -q 'controlled_by_grob-client' /etc/security/capability.conf
        then
            # we created and control this file, so lets remove it
            rm -f /etc/security/capability.conf
        else
            # not our file, just remove our line
            sed -ri '/^[[:blank:]]*cap_dac_read_search[[:blank:]]+%{grob_client_user}[[:blank:]]*$/ d' /etc/security/capability.conf
        fi
    fi

    # remove our user
    userdel --remove "%{grob_client_user}" >/dev/null 2>&1 || \
        echo "warning: failed to remove user %{grob_client_user}" >&2
fi

%triggerin -- borgbackup
setcap cap_dac_read_search+ie /usr/bin/borg

%files
%defattr(-,root,root)
%doc doc/*
%attr(0755,-,-) %{_bindir}/*
%attr(0755,-,-) %{_libexecdir}/grob-callback-handler

%files client
%defattr(-,root,root)
%doc doc/LICENSE
%attr(0755,-,-) %{_libexecdir}/%{name}/grob-client
%attr(0755,-,-) %{_libexecdir}/%{name}/grob-client-helper*
%attr(0750,%{grob_client_user},root) %{_libexecdir}/%{name}/grob-client-borg-cap-wrapper
